"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var isEmpty_1 = __importDefault(require("lodash/isEmpty"));
var has_own_prop_1 = __importDefault(require("has-own-prop"));
var hash_sum_1 = __importDefault(require("hash-sum"));
var convert_hex_1 = __importDefault(require("./utilities/convert-hex"));
function castString(input, stringifyBoolVerb) {
    if (stringifyBoolVerb === void 0) { stringifyBoolVerb = false; }
    var _return = input ? '' + input : '';
    if (typeof input === 'boolean') {
        if (stringifyBoolVerb) {
            _return = input ? 'true' : 'false';
        }
        else {
            _return = input ? '1' : '0';
        }
    }
    else if (typeof input === 'object') {
        if (isEmpty_1.default(input)) {
            _return = '';
        }
        else {
            _return = JSON.stringify(input);
        }
    }
    else if (input === 0) {
        _return = '0';
    }
    return _return;
}
function castNumber(input, hashObjects) {
    if (hashObjects === void 0) { hashObjects = false; }
    var match = /\d+/.exec(input);
    var isDigitString = match ? match[0] === input : false;
    var isArray = Array.isArray(input);
    var _hash = convert_hex_1.default(hash_sum_1.default(input));
    if (typeof input === 'string' && !isDigitString && input) {
        return _hash;
    }
    else if (input && typeof input === 'object') {
        if (hashObjects) {
            return _hash;
        }
        else {
            if (isArray) {
                return input.reduce(function (a, b) { return (castNumber(a) + castNumber(b)); }, 0);
            }
            else {
                return castNumber(Object.values(input));
            }
        }
    }
    else {
        return input ? +input : 0;
    }
}
function castBoolean(input) {
    if (typeof input === 'object' && isEmpty_1.default(input)) {
        return false;
    }
    else {
        return !!input;
    }
}
function castArray(input) {
    if (Array.isArray(input)) {
        return input;
    }
    else if (typeof input === 'object' && input) {
        return Object.values(input);
    }
    else if (input || typeof input === 'boolean' || input === 0) {
        return [input];
    }
    else {
        return [];
    }
}
function castObject(input, defaultKey) {
    var _a;
    var isArray = Array.isArray(input);
    if (!(input || typeof input === 'boolean' || input === 0)) {
        return {};
    }
    else if (typeof input === 'object' && !isArray) {
        return input;
    }
    else if (isArray) {
        return __assign({}, input);
    }
    else {
        return _a = {}, _a[defaultKey] = input, _a;
    }
}
var TypeInsurance = (function () {
    function TypeInsurance(input, options) {
        this.input = input;
        this.defaultKey = 'key';
        this.hashObjects = false;
        this.stringifyBoolsVerbatim = false;
        if (options) {
            if (has_own_prop_1.default(options, 'defaultKey')) {
                this.defaultKey = castString(options.defaultKey);
            }
            if (has_own_prop_1.default(options, 'hashObjects')) {
                this.hashObjects = options.hashObjects;
            }
            if (has_own_prop_1.default(options, 'stringifyBoolsVerbatim')) {
                this.stringifyBoolsVerbatim = options.stringifyBoolsVerbatim;
            }
        }
        this.string = this._string();
        this.number = this._number();
        this.boolean = this._boolean();
        this.array = this._array();
        this.object = this._object();
    }
    TypeInsurance.prototype._string = function () {
        var _a = this, input = _a.input, stringifyBoolsVerbatim = _a.stringifyBoolsVerbatim;
        return castString(input, stringifyBoolsVerbatim);
    };
    TypeInsurance.prototype._number = function () {
        var _a = this, input = _a.input, hashObjects = _a.hashObjects;
        return castNumber(input, hashObjects);
    };
    TypeInsurance.prototype._boolean = function () {
        var input = this.input;
        return castBoolean(input);
    };
    TypeInsurance.prototype._array = function () {
        var input = this.input;
        return castArray(input);
    };
    TypeInsurance.prototype._object = function () {
        var _a = this, input = _a.input, defaultKey = _a.defaultKey;
        return castObject(input, defaultKey);
    };
    return TypeInsurance;
}());
exports.default = TypeInsurance;
//# sourceMappingURL=index.js.map